################################################################################################################################
#Design
################################################################################################################################
#Default Libraries
import inspect

#Program
from .Design.N_Stock import Stock
from .Design.N_Element import Element
from .Design.N_Configure import Configure

class Design:
    def __init__(self, Global, Main):
        try:
            self.Global = Global
            self.Main = Main
            self.Widget = []
            self.Database = False
            self.Project_Path = False
            self.Project_Data = False
            self.Display_ID = False
            self.Alignment = 'Pixel'
            
            #Frame
            Fixture = self.Main.Frame.Locate(100, 100, 0, 0)
            self.Frame = self.Global['Gluonix'].Frame(self.Main.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Border_Size=0, Display=False)
            self.Frame.Config(Resize=False, Move=False)
            self.Frame.Create()
            self.Main.Widget.append(self.Frame)
            
            #Label
            Fixture = self.Frame.Locate(39, 10, 1, 0)
            self.Label = self.Global['Gluonix'].Label(self.Frame)
            self.Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label.Config(Border_Size=0, Display=True)
            self.Label.Config(Foreground='black', Font_Size=16, Font_Weight='normal', Align='w')
            self.Label.Config(Resize=True, Move=True)
            self.Label.Create()
            
                #Close
            Fixture = self.Frame.Locate(12, 6, 86, 2)
            self.Close = self.Global['Gluonix'].Label(self.Frame)
            self.Close.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Close.Config(Background='#e1e1e1', Foreground='black', Value='Close', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Close.Bind(On_Hover_In=lambda E: self.Close.Config(Border_Color='#0078d7', Background='#d5dcf0'))
            self.Close.Bind(On_Hover_Out=lambda E: self.Close.Config(Border_Color='#adadad', Background='#e1e1e1'))
            self.Close.Bind(On_Click=lambda E: self.Close_Project())
            self.Close.Create()
            
            #Widget View
            self.Stock = Stock(self.Global, self)
            
            #Display View
            self.Element = Element(self.Global, self)
            
            #Configure View
            self.Configure = Configure(self.Global, self)
                    
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Update(self, Loading=True):
        try:
            if Loading:
                self.Global['Loading'].Show()
                self.Global['GUI'].After(500, lambda:self.Update(Loading=False))
            else:
                if self.Display_ID and self.Project_Data:
                    if self.Database:
                        self.Database.Close()
                    self.Database = self.Global['Gluonix'].SQL(f'{self.Project_Path}/Data/NGD.dll')
                    Display_Data_Temp = self.Database.Get(f"SELECT * FROM `Display` WHERE `ID`='{self.Display_ID}'", Keys=True)
                    self.Display_Data = Display_Data_Temp[0]
                    self.Label.Set(self.Display_Data['Title'])
                    self.Alignment = self.Display_Data['Alignment']
                    setattr(self, self.Display_ID, self.Global['Gluonix'].Popup(self.Global['GUI']))
                    Temp_Root = getattr(self, self.Display_ID)
                    Temp_Root.Config(Width=int(self.Display_Data['Width']), Height=int(self.Display_Data['Height']), Left=int(self.Display_Data['Left']), Top=int(self.Display_Data['Top']))
                    Temp_Root.Config(Title=self.Display_Data['Title'], Background=self.Display_Data['Background'], Icon=f"{self.Project_Path}/Data/File/{self.Project_Data['Icon']}", Resizable=bool(int(self.Display_Data['Resizable'])))
                    Temp_Root.Config(Persistent=True, Full_Screen=bool(int(self.Display_Data['Full_Screen'])), Minimize=True, Toolbar=bool(int(self.Display_Data['Toolbar'])), Alignment=self.Display_Data['Alignment'], Menu_Enable=bool(int(self.Display_Data['Menu'])))
                    Temp_Root.Create()
                    if bool(int(self.Display_Data['Menu'])):
                        Temp_Root.Add_Menu(Name='File')
                        Temp_Root.Add_Menu(Name='Help')
                    self.Element.Default(Parent=self.Display_ID)
                    self.Configure.Hide_All()
                    self.Frame.Show()
                    Temp_Root.Show()
                self.Global['Loading'].Hide()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Close_Project(self):
        try:
            Temp_Root = getattr(self, self.Display_ID)
            Temp_Root.Close()
            if self.Database:
                self.Database.Close()
            self.Database = False
            self.Frame.Hide()
            self.Main.Home.Frame.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


################################################################################################################################
#Message
################################################################################################################################
import time
import inspect

class Message:
    def __init__(self, Global):
        try:
            self.Global = Global
            self.List = []
            #Frame 
            Fixture = self.Global['GUI'].Locate(25, 15, 73.5, 82.5)
            self.Frame = self.Global['Gluonix'].Frame(self.Global['GUI'])
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Border_Size=1, Display=False)
            self.Frame.Create()
            self.Global['Widget'].append(self.Frame)
                #Top Bar
            Fixture = self.Frame.Locate(100, 20, 0, 0)
            self.Top_Bar = self.Global['Gluonix'].Label(self.Frame)
            self.Top_Bar.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Top_Bar.Config(Background='#5a1b1b', Border_Size=0)
            self.Top_Bar.Create()
                #Close Label
            Fixture = self.Frame.Locate(7, 20, 93, 0)
            self.Close_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Close_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Close_Label.Config(Background='#5a1b1b', Foreground='white', Value="", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Close_Label.Bind(On_Click=lambda E: self.Hide(), On_Hover_In=lambda E: self.Close_Highlight_On(), On_Hover_Out=lambda E: self.Close_Highlight_Off())
            self.Close_Label.Create()
                #Close Image
            Fixture = self.Frame.Locate(5, 10, 94, 5)
            self.Close = self.Global['Gluonix'].Image(self.Frame)
            self.Close.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Close.Config(Background='#5a1b1b', Path=self.Global['Image']('Cross_White'), Border_Size=0)
            self.Close.Bind(On_Click=lambda E: self.Hide(), On_Hover_In=lambda E: self.Close_Highlight_On(), On_Hover_Out=lambda E: self.Close_Highlight_Off())
            self.Close.Create()
                #Status Image
            Fixture = self.Frame.Locate(20, 77, 4, 21)
            self.Image = self.Global['Gluonix'].Image(self.Frame)
            self.Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Image.Config(Path=self.Global['Image']('Success'), Border_Size=0)
            self.Image.Create()
                #Message Label
            Fixture = self.Frame.Locate(73, 77, 25, 21)
            self.Label = self.Global['Gluonix'].Label(self.Frame)
            self.Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label.Config(Foreground='black', Value="Message", Font_Size=9, Font_Weight='normal', Align='center', Border_Size=0)
            self.Label.Create()
            
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Close_Highlight_On(self):
        try:
            self.Close_Label.Config(Background='red')
            self.Close.Config(Background='red')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Close_Highlight_Off(self):
        try:
            self.Close_Label.Config(Background='#5a1b1b')
            self.Close.Config(Background='#5a1b1b')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))

    def Show(self, Type='Success', Text=''):
        try:
            self.Label.Config(Value=Text)
            self.Image.Config(Path=self.Global['Image'](Type.capitalize()))
            self.Frame.Show()
            if Type.lower()=='warning' or Type.lower()=='error':
                Time = int(time.time())
                self.List.append([Type, Text, Time])
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Hide(self, Delay=False):
        try:
            if Delay:
                self.Global['GUI'].After(Delay*1000, lambda: self.Hide(Delay=False))
            else:
                self.Frame.Hide()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Clear(self):
        try:
            self.List = []
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


# IMPORT LIBRARIES
import math
import tkinter as TK
from .N_GUI import GUI
from .N_Frame import Frame
from .N_Custom import Event_Bind

class Entry:

    def __init__(self, Main, *args, **kwargs):
        self._GUI = GUI._Instance
        if self._GUI is not None:
            self._Type = "Entry"
            try:
                self._Config = ['Name', 'Auto_Dark', 'Background', 'Light_Background', 'Dark_Background', 'Foreground', 'Light_Foreground', 'Dark_Foreground', 'Border_Color', 'Light_Border_Color', 'Dark_Border_Color', 'Border_Size', 'Resize_Width', 'Resize', 'Resize_Height', 'Move', 'Move_Left', 'Move_Top', 'Popup', 'Display', 'Left', 'Top', 'Width', 'Height', 'Font_Size', 'Font_Weight', 'Font_Family','Align', 'Disable', 'Secure', 'Disable_Background', 'Disable_Foreground', 'Select_Background', 'Select_Foreground', 'Hover_Background', 'Light_Hover_Background', 'Dark_Hover_Background', 'Hover_Foreground', 'Light_Hover_Foreground', 'Dark_Hover_Foreground', 'Hover_Border_Color', 'Light_Hover_Border_Color', 'Dark_Hover_Border_Color']
                self._Initialized = False
                self._Widget = []
                self._Name = False
                self._Last_Name = False
                self._Resize_Font, self._Resize, self._Resize_Width, self._Resize_Height, self._Move, self._Move_Left, self._Move_Top = True, True, True, True, True, True, True
                self._Popup = False
                self._Display = True
                self._Main = Main
                self._Frame = Frame(self._Main)
                self._Widget = TK.Entry(self._Frame._Frame)
                self._Border_Color = '#000000'
                self._Border_Size = 0
                self._Background = self._Main._Background
                self._Background_Main = True
                self._Foreground = '#000000'
                self._Disable_Foreground = self._Foreground
                self._Disable_Background = '#d0d3d4'
                self._Select_Foreground = self._Background
                self._Select_Background = self._Foreground
                self._Hover_Background = False
                self._Hover_Foreground = False
                self._Hover_Border_Color = False
                self._Last_Background = False
                self._Last_Foreground = False
                self._Last_Border_Color = False
                self._Font_Size = 12
                self._Font_Weight = 'normal'
                self._Font_Family = 'Helvetica'
                self._Align = 'center'
                self._Disable = False
                self._Secure = False
                self._Resizable = self._Main._Resizable
                self._Auto_Dark = True
                self._On_Show = False
                self._On_Hide = False
                self._On_Hover_In = False
                self._On_Hover_Out = False
            except Exception as E:
                self._GUI.Error(f"{self._Type} -> Init -> {E}")
        else:
            print("Error: Gluonix -> GUI Instance Has Not Been Created")

    def __str__(self):
        return "Nucleon_Glunoix_Entry[]"

    def __repr__(self):
        return "Nucleon_Glunoix_Entry[]"
    
    def Copy(self, Name=False, Main=False):
        try:
            if not Main:
                Main = self._Main
            Instance = type(self)(Main)
            for Key in self._Config:
                setattr(Instance, "_"+Key, getattr(self, "_"+Key))
            if Name:
                setattr(Instance, "_Name", Name)
            Instance.Create()
            return Instance
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Copy -> {E}")
        
    def Delete(self):
        try:
            self._Main._Widget.remove(self)
            self._Widget.destroy()
            self._Frame.Delete()
            if self:
                del self
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Delete -> {E}")
            
    def Hide(self):
        try:
            self._Frame.Hide()
            self._Display = False
            if self._On_Hide:
                self._On_Hide()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Hide -> {E}")
            
    def Show(self):
        try:
            self._Display = True
            if self._Resizable:
                self.Resize()
            else:
                self.Display()
            if self._On_Show:
                self._On_Show()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Show -> {E}")
            
    def Display(self):
        try:
            self._Frame.Show()
            self._Widget.place(x=0, y=0, width=self._Width_Current-(self._Border_Size*2), height=self._Height_Current-(self._Border_Size*2))
            self._Display = True
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Display -> {E}")
            
    def Focus(self):
        try:
            self._Widget.focus_set()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Focus -> {E}")
    
    def Grab(self, Path=False):
        try:
            return self._GUI.Grab_Widget(Path=Path, Widget=self)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Grab -> {E}")
            
    def Get(self):
        try:
            return self._Widget.get()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Get -> {E}")
            
    def Set(self, Value):
        try:
            self._Widget.config(state=TK.NORMAL)
            self._Widget.delete(0, TK.END)
            self._Widget.insert(0, Value)
            if self._Disable:
                State = TK.DISABLED
            else:
                State = TK.NORMAL
            self._Widget.config(state=State)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Set -> {E}")
            
    def Widget(self):
        try:
            return self._Widget
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Widget -> {E}")
            
    def Bind(self, **Input):
        try:
            if 'On_Show' in Input:
                self._On_Show = Input['On_Show']
            if 'On_Hide' in Input:
                self._On_Hide = Input['On_Hide']
            self._Frame.Bind(**Input)
            if 'On_Hover_In' in Input:
                self._On_Hover_In = Input['On_Hover_In']
            Input['On_Hover_In'] = lambda E: self.On_Hover_In(E)
            if 'On_Hover_Out' in Input:
                self._On_Hover_Out = Input['On_Hover_Out']
            Input['On_Hover_Out'] = lambda E: self.On_Hover_Out(E)
            Event_Bind(self._Widget, **Input)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Bind -> {E}")
            
    def On_Hover_In(self, E):
        try:
            Config = {}
            if self._Hover_Background:
                self._Last_Background = self._Background
                Config['Background'] = self._Hover_Background
            if self._Hover_Foreground:
                self._Last_Foreground = self._Foreground
                Config['Foreground'] = self._Hover_Foreground
            if self._Hover_Border_Color:
                self._Last_Border_Color = self._Border_Color
                Config['Border_Color'] = self._Hover_Border_Color
            if len(Config)>0:
                self.Config(**Config)
            if self._On_Hover_In:
                self._On_Hover_In(E)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> On_Hover_In -> {E}")
            
    def On_Hover_Out(self, E):
        try:
            Config = {}
            if self._Hover_Background and self._Last_Background:
                Config['Background'] = self._Last_Background if self._Background==self._Hover_Background else self._Background
            if self._Hover_Foreground and self._Last_Foreground:
                Config['Foreground'] = self._Last_Foreground if self._Foreground==self._Hover_Foreground else self._Foreground
            if self._Hover_Border_Color and self._Last_Border_Color:
                Config['Border_Color'] = self._Last_Border_Color if self._Border_Color==self._Hover_Border_Color else self._Border_Color
            if len(Config)>0:
                self.Config(**Config)
            if self._On_Hover_Out:
                self._On_Hover_Out(E)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> On_Hover_Out -> {E}")
            
    def Config_Get(self, *Input):
        try:
            Return = {}
            for Each in self._Config:
                if Each in Input:
                    Return[Each] = getattr(self, "_"+Each)
            return Return
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Config_Get -> {E}")
                
    def Config(self, **Input):
        try:
            Run = False
            for Each in self._Config:
                if Each in Input:
                    Value = Input[Each]
                    setattr(self, "_"+Each, Value)
                    Run = True
            self._Frame.Config(**Input)
            if self._Initialized and Run:
                self.Create()
            if "Background" in Input:
                self._Background_Main = not bool(Input["Background"])
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Config -> {E}")
            
    def Position(self, Left=None, Top=None):
        try:
            if Left is not None:
                self._Left = Left
            if Top is not None:
                self._Top = Top
            if Left is not None or Top is not None:
                self._Frame.Position(Left=Left, Top=Top)
                self.Relocate()
            return self._Frame.Position()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Position -> {E}")
            
    def Size(self, Width=False, Height=False):
        try:
            if Width:
                self._Width = Width
            if Height:
                self._Height = Height
            if Width or Height:
                self._Frame.Size(Width=Width, Height=Height)
                self.Relocate()
            return self._Frame.Size()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Size -> {E}")
        
    def Locate(self, Width, Height, Left, Top):
        try:
            Width = self._Width*(Width/100)
            Height = self._Height*(Height/100)
            Left = self._Width*(Left/100)-self._Border_Size
            Top = self._Height*(Top/100)-self._Border_Size
            return [Width, Height, Left, Top]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Locate -> {E}")
        
    def Locate_Reverse(self, Width, Height, Left, Top):
        try:
            Width = round((Width/self._Width)*100, 3)
            Height = round((Height/self._Height)*100, 3)
            Left =  round((Left/self._Width)*100, 3)
            Top =  round((Top/self._Height)*100, 3)
            return [Width, Height, Left, Top]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Locate_Reverse -> {E}")
            
    def Create(self):
        try:
            if not self._Background:
                self._Background = self._Main._Background
                if not hasattr(self, "_Light_Background"):
                    setattr(self, "_Light_Background", self._Background)
                if not hasattr(self, "_Dark_Background"):
                    setattr(self, "_Dark_Background", self._GUI.Invert(self._Background))
            if self._Auto_Dark:
                self.Update_Color()
            if not self._Initialized:
                self._Width_Current, self._Height_Current, self._Left_Current, self._Top_Current, self._Font_Size_Current = self._Width, self._Height, self._Left, self._Top, self._Font_Size
                self._Frame.Config(Width=self._Width_Current, Height=self._Height_Current, Left=self._Left_Current, Top=self._Top_Current)
                self._Frame.Config(Background=self._Background, Border_Size=self._Border_Size, Border_Color=self._Border_Color)
                self._Frame.Create()
                Event_Bind(self._Widget, On_Hover_In=lambda E: self.On_Hover_In(E), On_Hover_Out=lambda E: self.On_Hover_Out(E))
                if not self._Display:
                    self.Hide()
                self._Main._Widget.append(self)
                self._Initialized = True
            if self._Disable:
                State = TK.DISABLED
            else:
                State = TK.NORMAL
            if self._Secure:
                Show = '*'
            else:
                Show = ''
            self.Font()
            self._Font = TK.font.Font(family=self._Font_Family, size=self._Font_Size_Current, weight=self._Font_Weight)
            self._Widget.config(background=self._Background, foreground=self._Foreground, font=self._Font, state=State, show=Show, justify=self._Align, bd=0, highlightthickness=0, relief=TK.FLAT)
            self._Widget.config(disabledbackground=self._Disable_Background, disabledforeground=self._Disable_Foreground, selectbackground=self._Select_Background, selectforeground=self._Select_Foreground)
            self.Resize()
            if self._Name!=self._Last_Name:
                if self._Last_Name:
                    if self._Last_Name in self._Main.__dict__:
                        del self._Main.__dict__[self._Last_Name]
                if self._Name:
                    self._Main.__dict__[self._Name] = self
                self._Last_Name = self._Name
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Create -> {E}")
            
    def Update_Color(self):
        try:
            self._GUI.Initiate_Colors(self)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Update_Color -> {E}")
            
    def Font(self):
        try:
            if self._Resize_Font:
                Width_Ratio = self._Frame._Width_Current / self._Frame._Width
                Height_Ratio = self._Frame._Height_Current / self._Frame._Height
                if Width_Ratio < Height_Ratio:
                    self._Font_Size_Current = math.floor(self._Font_Size * Width_Ratio)
                else:
                    self._Font_Size_Current = math.floor(self._Font_Size * Height_Ratio)
            else:
                self._Font_Size_Current = self._Font_Size
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Font -> {E}")
            
    def Adjustment(self):
        try:
            Width_Difference = self._Main._Width_Current - self._Main._Width
            Width_Ratio = self._Width / (self._Main._Width - self._Main._Border_Size*2)
            self._Width_Adjustment = Width_Difference * Width_Ratio
            Height_Difference = self._Main._Height_Current - self._Main._Height
            Height_Ratio = self._Height / (self._Main._Height - self._Main._Border_Size*2)
            self._Height_Adjustment = Height_Difference * Height_Ratio
            Left_Ratio = self._Left / self._Main._Width
            self._Left_Adjustment = Width_Difference * Left_Ratio
            Top_Ratio = self._Top / self._Main._Height
            self._Top_Adjustment = Height_Difference * Top_Ratio
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Adjustment -> {E}")
            
    def Relocate(self, Direct=False):
        try:
            self.Adjustment()
            if Direct or (self._Resize and self._Resize_Width):
                self._Width_Current = self._Width + self._Width_Adjustment
            else:
                self._Width_Current = self._Width
            if Direct or (self._Resize and self._Resize_Height):
                self._Height_Current = self._Height + self._Height_Adjustment
            else:
                self._Height_Current = self._Height
            if Direct or (self._Move and self._Move_Left):
                self._Left_Current = self._Left + self._Left_Adjustment
            else:
                self._Left_Current = self._Left
            if Direct or (self._Move and self._Move_Top):
                self._Top_Current = self._Top + self._Top_Adjustment
            else:
                self._Top_Current = self._Top
            if self._Display:
                self._Font = TK.font.Font(family=self._Font_Family, size=self._Font_Size_Current, weight=self._Font_Weight)
                self._Widget.config(font=self._Font)
                self.Display()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Relocate -> {E}")
            
    def Resize(self):
        try:
            self.Font()
            self.Relocate()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Resize -> {E}")


################################################################################################################################
#Menu
################################################################################################################################
import os
import inspect
import requests
import _thread
import sqlite3

#Program
from .N_Project_New import New

class Project:
    def __init__(self, Global, Home):
        try:
            self.Global = Global
            self.Home = Home
            self.Widget = []
            
            Fixture = self.Home.Frame.Locate(20, 100, 0, 0)
            self.Frame = self.Global['Gluonix'].Scroll(self.Home.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='white', Border_Size=0, Display=True) 
            self.Frame.Config(Resize=True, Move=True)
            self.Frame.Create()
            
            #Project View
                #Label
            Fixture = self.Frame.Locate(70, 4, 13, 2)
            self.Project_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Project_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_Label.Config(Foreground='gray', Value="Project", Font_Size=16, Font_Weight='normal', Align='w', Border_Size=0)
            self.Project_Label.Create()
            
                #New
                    #Image
            Fixture = self.Frame.Locate(15, 4, 10, 8)
            self.New_Image = self.Global['Gluonix'].Image(self.Frame)
            self.New_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.New_Image.Config(Path=self.Global['Image']('New'), Border_Size=0)
            self.New_Image.Bind(On_Click=lambda E: self.Create_Project(), Cursor_Hand=True)
            self.New_Image.Create()
                    #Label
            Fixture = self.Frame.Locate(70, 4, 25, 8)
            self.New_Label = self.Global['Gluonix'].Label(self.Frame)
            self.New_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.New_Label.Config(Foreground='black', Value="New project", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.New_Label.Bind(On_Click=lambda E: self.Create_Project(), Cursor_Hand=True)
            self.New_Label.Create()
            
                #Open
                    #Image
            Fixture = self.Frame.Locate(15, 4, 10, 12)
            self.Open_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Open_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Open_Image.Config(Path=self.Global['Image']('Open'), Border_Size=0)
            self.Open_Image.Bind(On_Click=lambda E: self.Open_Project(), Cursor_Hand=True)
            self.Open_Image.Create()
                    #Label
            Fixture = self.Frame.Locate(70, 4, 25, 12)
            self.Open_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Open_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Open_Label.Config(Foreground='black', Value="Open project", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Open_Label.Bind(On_Click=lambda E: self.Open_Project(), Cursor_Hand=True)
            self.Open_Label.Create()
            
                #Version
                    #Current
            Fixture = self.Frame.Locate(17, 4, 10, 90)
            self.Current_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Current_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Current_Label.Config(Foreground='black', Value=f"V {self.Global['Version']}.{self.Global['Revision']}", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Current_Label.Create()
            
            #Create New Project
            self.New = New(self.Global, self)
                   
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Create_Project(self):
        try:
            self.New.Reset()
            self.New.Frame.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Open_Project(self):
        try:
            Project_File_Path = self.Global['GUI'].File(Initial=os.path.join(os.path.expanduser('~'), 'Documents'), Title='Select Project', Default='.ng', Type=[["Nucleon Gluonix (*.ng)", "*.ng"]])
            if Project_File_Path:
                Project_Path = os.path.dirname(Project_File_Path)
                self.Update_Database(f'{Project_Path}/Data/NGD.dll', self.Global['Data']('NGD.dll'))
                self.Home.Panel.Overview.Project_Path = Project_Path
                self.Home.Panel.Overview.Update()
                self.Global['Message'].Hide()
        except Exception as E:
            self.Global['Message'].Show('Error', 'Project Files Are Corrupted')
            self.Project_Path = False
            self.Project_Name = False
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Database(self, Old_Structure_Database, New_Structure_Database):
        Conn_Old = sqlite3.connect(Old_Structure_Database)
        Conn_New = sqlite3.connect(New_Structure_Database)
        Cursor_Old = Conn_Old.cursor()
        Cursor_New = Conn_New.cursor()
        Cursor_New.execute("SELECT name FROM sqlite_master WHERE type='table';")
        Tables = Cursor_New.fetchall()
        for (Table,) in Tables:
            Cursor_New.execute(f"PRAGMA table_info({Table});")
            Columns_New = Cursor_New.fetchall()
            Cursor_Old.execute(f"PRAGMA table_info({Table});")
            Columns_Old = Cursor_Old.fetchall()
            if Columns_Old:
                Existing_Columns = {col[1] for col in Columns_Old}
                New_Columns = {col[1]: col[4] for col in Columns_New}
                for Col in Columns_New:
                    Column_Name = Col[1]
                    Column_Type = Col[2]
                    Default_Value = Col[4]
                    if Column_Name not in Existing_Columns:
                        if Default_Value is None:
                            Alter_Statement = f"ALTER TABLE {Table} ADD COLUMN {Column_Name} {Column_Type};"
                        else:
                            Alter_Statement = f"ALTER TABLE {Table} ADD COLUMN {Column_Name} {Column_Type} DEFAULT {Default_Value};"
                        Cursor_Old.execute(Alter_Statement)
            else:
                Create_Statement = f"CREATE TABLE {Table} (" + ", ".join([f"{col[1]} {col[2]} DEFAULT {col[4]}" for col in Columns_New]) + ");"
                Cursor_Old.execute(Create_Statement)
        Conn_Old.commit()


################################################################################################################################
#Configure_Text
################################################################################################################################
import inspect
from tkinter import colorchooser

class Configure_Canvas_Text:
    def __init__(self, Global, Configure):
        try:
            self.Global = Global
            self.Configure = Configure
            self.Widget = []
            self.ID = False
            self.Root = False
            self.Root_ID = False
            self.Element = False
            
            #Frame
            Fixture = self.Configure.Frame.Locate(100, 95, 0, 5)
            self.Frame = self.Global['Gluonix'].Scroll(self.Configure.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='#FFFFFF', Border_Size=0, Display=False)
            self.Frame.Config(Resize=False, Move=False)
            self.Frame.Create()
            self.Configure.Widget.append(self)
            
            #Name Label
            Fixture = self.Frame.Locate(25, 5, 3, 2)
            self.Name_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Name_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Name_Label.Config(Foreground='#000000', Value="Name:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Name_Label.Create()
            
            #Name Entry
            Fixture = self.Frame.Locate(60, 5, 28, 2)
            self.Name_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Name_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Name_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='left', Border_Size=1)
            self.Name_Entry.Bind(On_Key_Release=lambda E: self.Update_Name())
            self.Name_Entry.Create()
            
            #Color Label
            Fixture = self.Frame.Locate(25, 5, 3, 9)
            self.Color_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Color_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Color_Label.Config(Foreground='#000000', Value="Color:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Color_Label.Create()
            
            #Color Color
            Fixture = self.Frame.Locate(7, 5, 28, 9)
            self.Color_Color = self.Global['Gluonix'].Label(self.Frame)
            self.Color_Color.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Color_Color.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='w', Border_Size=1)
            self.Color_Color.Bind(On_Click=lambda E: self.Select_Color(self.Color_Color))
            self.Color_Color.Bind(On_Change=lambda : self.Update_Color())
            self.Color_Color.Create()
            
            #Alignment Lock
            Fixture = self.Frame.Locate(7, 5, 37, 9)
            self.Lock = False
            self.Lock_Image = {True: 'Lock_Closed', False: 'Lock_Open'}
            self.Alignment_Lock = self.Global['Gluonix'].Image(self.Frame)
            self.Alignment_Lock.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Alignment_Lock.Config(Border_Size=0)
            self.Alignment_Lock.Bind(On_Click=lambda E: self.Update_Lock())
            self.Alignment_Lock.Create()
            
            #Thickness Label
            Fixture = self.Frame.Locate(25, 5, 3, 16)
            self.Thickness_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Thickness_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Thickness_Label.Config(Foreground='#000000', Value="Thickness:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Thickness_Label.Create()
            
            #Thickness Entry
            Fixture = self.Frame.Locate(40, 5, 28, 16)
            self.Thickness_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Thickness_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Thickness_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Thickness_Entry.Bind(On_Key_Release=lambda E: self.Update_Thickness())
            self.Thickness_Entry.Create()
            
            #Width Label
            Fixture = self.Frame.Locate(25, 5, 3, 23)
            self.Width_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Width_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Width_Label.Config(Foreground='#000000', Value="Width:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Width_Label.Create()
            
            #Width Entry
            Fixture = self.Frame.Locate(40, 5, 28, 23)
            self.Width_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Width_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Width_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Width_Entry.Bind(On_Key_Release=lambda E: self.Update_Width())
            self.Width_Entry.Create()
            
            #Height Label
            Fixture = self.Frame.Locate(25, 5, 3, 30)
            self.Height_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Height_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Height_Label.Config(Foreground='#000000', Value="Height:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Height_Label.Create()
            
            #Height Entry
            Fixture = self.Frame.Locate(40, 5, 28, 30)
            self.Height_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Height_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Height_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Height_Entry.Bind(On_Key_Release=lambda E: self.Update_Height())
            self.Height_Entry.Create()
            
            #Left Label
            Fixture = self.Frame.Locate(25, 5, 3, 37)
            self.Left_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Left_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Left_Label.Config(Foreground='#000000', Value="Left:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Left_Label.Create()
            
            #Left Entry
            Fixture = self.Frame.Locate(40, 5, 28, 37)
            self.Left_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Left_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Left_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Left_Entry.Bind(On_Key_Release=lambda E: self.Update_Left())
            self.Left_Entry.Create()
            
            #Top Label
            Fixture = self.Frame.Locate(25, 5, 3, 44)
            self.Top_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Top_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Top_Label.Config(Foreground='#000000', Value="Top:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Top_Label.Create()
            
            #Top Entry
            Fixture = self.Frame.Locate(40, 5, 28, 44)
            self.Top_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Top_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Top_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Top_Entry.Bind(On_Key_Release=lambda E: self.Update_Top())
            self.Top_Entry.Create()
            
            #Resize Label
            Fixture = self.Frame.Locate(25, 5, 3, 51)
            self.Resize_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Resize_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Label.Config(Foreground='#000000', Value="Resize Width:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Resize_Label.Create()
            
            #Resize Check
            Fixture = self.Frame.Locate(7, 5, 27.7, 51)
            self.Resize_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Resize_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Check.Config(Border_Size=0)
            self.Resize_Check.Bind(On_Change=lambda : self.Update_Resize())
            self.Resize_Check.Create()
            
            #Move Label
            Fixture = self.Frame.Locate(25, 5, 3, 58)
            self.Move_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Move_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Move_Label.Config(Foreground='#000000', Value="Move Left:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Move_Label.Create()
            
            #Move Check
            Fixture = self.Frame.Locate(7, 5, 27.7, 58)
            self.Move_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Move_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Move_Check.Config(Border_Size=0)
            self.Move_Check.Bind(On_Change=lambda : self.Update_Move())
            self.Move_Check.Create()
            
            #Value Label
            Fixture = self.Frame.Locate(25, 5, 3, 65)
            self.Value_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Value_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Value_Label.Config(Foreground='#000000', Value="Value:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Value_Label.Create()
            
            #Value Entry
            Fixture = self.Frame.Locate(60, 5, 28, 65)
            self.Value_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Value_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Value_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='left', Border_Size=1)
            self.Value_Entry.Bind(On_Key_Release=lambda E: self.Update_Value())
            self.Value_Entry.Create()
            
            #Size Label
            Fixture = self.Frame.Locate(25, 5, 3, 72)
            self.Size_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Size_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Size_Label.Config(Foreground='#000000', Value="Size:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Size_Label.Create()
            
            #Size Entry
            Fixture = self.Frame.Locate(40, 5, 28, 72)
            self.Size_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Size_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Size_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Size_Entry.Bind(On_Key_Release=lambda E: self.Update_Size())
            self.Size_Entry.Create()
            
            #Weight Label
            Fixture = self.Frame.Locate(25, 5, 3, 79)
            self.Weight_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Weight_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Weight_Label.Config(Foreground='#000000', Value="Weight:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Weight_Label.Create()
            
            #Weight Select
            Fixture = self.Frame.Locate(40, 5, 28, 79)
            self.Weight_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Weight_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Weight_Select.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Weight_Select.Add('normal')
            self.Weight_Select.Add('bold')
            self.Weight_Select.Bind(On_Change=lambda E: self.Update_Weight())
            self.Weight_Select.Create()
            
            #Font Label
            Fixture = self.Frame.Locate(25, 5, 3, 86)
            self.Font_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Font_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Font_Label.Config(Foreground='#000000', Value="Font:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Font_Label.Create()
            
            #Font Select
            Fixture = self.Frame.Locate(40, 5, 28, 86)
            self.Font_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Font_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Font_Select.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Font_Select.Add('Arial')
            self.Font_Select.Add('Courier')
            self.Font_Select.Add('Courier New')
            self.Font_Select.Add('Georgia')
            self.Font_Select.Add('Lucida Console')
            self.Font_Select.Add('Symbol')
            self.Font_Select.Add('Times New Roman')
            self.Font_Select.Add('Verdana')
            self.Font_Select.Add('Comic Sans MS')
            self.Font_Select.Add('Helvetica')
            self.Font_Select.Bind(On_Change=lambda E: self.Update_Font())
            self.Font_Select.Create()
            
            #Update Scroll
            self.Frame.Update(self.Font_Select)
            
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load(self, ID=False):
        try:
            if ID!=self.ID:
                self.Root = False
                self.Root_ID = False
                self.Element = False
                self.ID = ID
                Widget_Data = self.Configure.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{self.ID}'", Keys=True)
                if len(Widget_Data)==1:
                    Widget_Data = Widget_Data[0]
                    self.Root_ID = Widget_Data['Root']
                    self.Name_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Width_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Height_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Left_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Top_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Size_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Name_Label.Set(Widget_Data['Type'])
                    self.Name_Entry.Set(Widget_Data['Name'])
                    self.Color_Color.Config(Background=Widget_Data['Color'])
                    self.Lock = bool(Widget_Data['Lock'])
                    self.Alignment_Lock.Set(self.Global['Image'](self.Lock_Image[self.Lock]))
                    self.Thickness_Entry.Set(Widget_Data['Thickness'])
                    self.Width_Entry.Set(Widget_Data['Width'])
                    self.Height_Entry.Set(Widget_Data['Height'])
                    self.Left_Entry.Set(Widget_Data['Left'])
                    self.Top_Entry.Set(Widget_Data['Top'])
                    self.Resize_Check.Set(bool(Widget_Data['Resize']))
                    self.Move_Check.Set(bool(Widget_Data['Move']))
                    self.Value_Entry.Set(Widget_Data['Value'])
                    self.Weight_Select.Set(Widget_Data['Weight'])
                    self.Size_Entry.Set(Widget_Data['Size'])
                    self.Font_Select.Set(Widget_Data['Font'])
                    self.Configure.Hide_All()
                    self.Configure.Current = self
                    self.Frame.Show()
                    Temp_Root = Widget_Data['Root']
                    Root = Temp_Root
                    while Temp_Root!='Root':
                        Widget_Data = self.Configure.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root}'", Keys=True)
                        if len(Widget_Data)==1:
                            Widget_Data = Widget_Data[0]
                            Temp_Root = Widget_Data['Root']
                            Root = Temp_Root+'.'+Root
                        else:
                            break
                    Root_Split = Root.split('.')
                    Current = ''
                    for Each in Root_Split:
                        if Current:
                            Current = Current+'.'+Each
                        else:
                            Current = Each
                        if Current!='Root':
                            Temp_Root = self.Global['Custom'].Get_Attr_Class(self.Configure.Design, Current)
                            Temp_Root.Show()
                    self.Root = self.Global['Custom'].Get_Attr_Class(self.Configure.Design, Root)
                    self.Element = getattr(self.Root, self.ID)
                    self.Element.Show()
                else:
                    self.Global['Message'].Show('Error', 'Project Files Corrupted\nReopen Project')
            else:
                self.Element.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        self.Global['Loading'].Hide()
            
    def Movement_Update(self):
        try:
            if self.ID:
                Widget_Data = self.Configure.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{self.ID}'", Keys=True)
                Widget_Data = Widget_Data[0]
                self.Width_Entry.Set(Widget_Data['Width'])
                self.Height_Entry.Set(Widget_Data['Height'])
                self.Left_Entry.Set(Widget_Data['Left'])
                self.Top_Entry.Set(Widget_Data['Top'])
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Select_Color(self, Element):
        try:
            Color = colorchooser.askcolor(color=Element.Config_Get('Background')['Background'], title ="Choose Background Color")[1]
            if Color is not None:
                Element.Config(Background=Color)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Name(self):
        try:
            Name = self.Name_Entry.Get()
            TempName = self.Configure.Design.Database.Get(f"SELECT * FROM `Item` WHERE (`ID`!='{self.ID}' AND `Root`=='{self.Root_ID}' AND `Name`='{Name}')")
            if self.Global['Custom'].Valid_Variable(Name) and len(TempName)==0:
                self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Name`='{Name}' WHERE `ID`='{self.ID}'")
                self.Configure.Design.Element.Tree.Edit(Name=f' {Name}')
                self.Name_Entry.Config(Border_Color='#000000', Border_Size=1)
            else:
                self.Name_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Color(self):
        try:
            Color = self.Color_Color.Config_Get('Background')['Background']
            self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Color`='{Color}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Color=Color)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Lock(self):
        try:
            self.Lock = not self.Lock
            self.Alignment_Lock.Set(self.Global['Image'](self.Lock_Image[self.Lock]))
            self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Lock`='{int(self.Lock)}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Lock = self.Lock
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Thickness(self):
        try:
            Thickness = self.Thickness_Entry.Get()
            if self.Global['Custom'].Is_Float(Thickness):
                Thickness = float(Thickness)
                self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Thickness`='{Thickness}' WHERE `ID`='{self.ID}'")
                self.Thickness_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    self.Element.Config(Thickness=Thickness)
            else:
                self.Thickness_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Width(self):
        try:
            Width = self.Width_Entry.Get()
            if self.Global['Custom'].Is_Float(Width):
                Width = float(Width)
                self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Width`='{Width}' WHERE `ID`='{self.ID}'")
                self.Width_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    Fixture = [Width, 0, 0, 0]
                    self.Element.Config(Width=Fixture[0])
            else:
                self.Width_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Height(self):
        try:
            Height = self.Height_Entry.Get()
            if self.Global['Custom'].Is_Float(Height):
                Height = float(Height)
                self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Height`='{Height}' WHERE `ID`='{self.ID}'")
                self.Height_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    Fixture = [0, Height, 0, 0]
                    self.Element.Config(Height=Fixture[1])
            else:
                self.Height_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Left(self):
        try:
            Left = self.Left_Entry.Get()
            if self.Global['Custom'].Is_Float(Left):
                Left = float(Left)
                self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Left`='{Left}' WHERE `ID`='{self.ID}'")
                self.Left_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    Fixture = [0, 0, Left, 0]
                    self.Element.Config(Left=Fixture[2])
            else:
                self.Left_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Top(self):
        try:
            Top = self.Top_Entry.Get()
            if self.Global['Custom'].Is_Float(Top):
                Top = float(Top)
                self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Top`='{Top}' WHERE `ID`='{self.ID}'")
                self.Top_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    Fixture = [0, 0, 0, Top]
                    self.Element.Config(Top=Fixture[3])
            else:
                self.Top_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Resize(self):
        try:
            Resize = self.Resize_Check.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Resize`='{int(Resize)}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Resize=Resize)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Move(self):
        try:
            Move = self.Move_Check.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Move`='{int(Move)}' WHERE `ID`='{self.ID}'")
            if self.Element:
                self.Element.Config(Move=Move)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Value(self):
        try:
            Value = self.Value_Entry.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Value`='{Value}' WHERE `ID`='{self.ID}'")
            self.Element.Set(Value)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Weight(self):
        try:
            Weight = self.Weight_Select.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Weight`='{Weight}' WHERE `ID`='{self.ID}'")
            self.Element.Config(Weight=Weight)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Size(self):
        try:
            Size = self.Size_Entry.Get()
            if Size.isdigit():
                Size = int(Size)
                self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Size`='{Size}' WHERE `ID`='{self.ID}'")
                self.Size_Entry.Config(Border_Color='#000000', Border_Size=1)
                if self.Element:
                    self.Element.Config(Size=Size)
            else:
                self.Size_Entry.Config(Border_Color='red', Border_Size=2)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Font(self):
        try:
            Font = self.Font_Select.Get()
            self.Configure.Design.Database.Post(f"UPDATE `Item` SET `Font`='{Font}' WHERE `ID`='{self.ID}'")
            self.Element.Config(Font=Font)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


################################################################################################################################
#Home
################################################################################################################################
#Default Libraries
import inspect

#Program
from .Home.N_Project import Project
from .Home.N_Panel import Panel

class Home:
    def __init__(self, Global, Main):
        try:
            self.Global = Global
            self.Main = Main
            self.Widget = []
            
            #Frame
            Fixture = self.Main.Frame.Locate(100, 100, 0, 0)
            self.Frame = self.Global['Gluonix'].Frame(self.Main.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Border_Size=0, Display=True) 
            self.Frame.Config(Resize=True, Move=False)
            self.Frame.Create()
            self.Main.Widget.append(self.Frame)
            
            #Project View
            self.Project = Project(self.Global, self)
            
            #Panel View
            self.Panel = Panel(self.Global, self)
                    
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


################################################################################################################################
#Menu
################################################################################################################################
import os
import math
import shutil
import inspect
from tkinter import colorchooser

class Overview:
    def __init__(self, Global, Panel):
        try:
            self.Global = Global
            self.Panel = Panel
            self.Widget = []
            self.Project_Path = False
            self.Runtime_Path = False
            self.TempDatabase = False
            self.Launch = False
            self.Direct = False
            self.Resize = False
            
            Fixture = self.Panel.Frame.Locate(100, 100, 0, 0)
            self.Frame = self.Global['Gluonix'].Frame(self.Panel.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Border_Size=0, Display=False)
            self.Frame.Create()
            self.Panel.Widget.append(self.Frame)
            
                #Info Label
            Fixture = self.Frame.Locate(35, 8, 3, 5)
            self.Info_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Info_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Info_Label.Config(Foreground='black', Value="Project Configuration:", Font_Size=14, Font_Weight='bold', Align='w', Border_Size=0)
            self.Info_Label.Create()
            
                #Display Label
            Fixture = self.Frame.Locate(8, 4, 63, 7)
            self.Display_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Display_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Display_Label.Config(Foreground='black', Value="Display:", Font_Size=14, Font_Weight='bold', Align='w', Border_Size=0)
            self.Display_Label.Create()
            
                #Display Select
            Fixture = self.Frame.Locate(15, 4, 72, 7)
            self.Display_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Display_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Display_Select.Config(Foreground='black', Font_Size=11, Font_Weight='normal', Align='center', Border_Size=1)
            self.Display_Select.Bind(On_Change=lambda E: self.Update(Partial=True))
            self.Display_Select.Create()
            
                #Add Image
            Fixture = self.Frame.Locate(3, 4, 88, 7)
            self.Add_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Add_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Add_Image.Config(Background=False, Border_Size=0, Hand_Cursor=True)
            self.Add_Image.Config(Path=self.Global['Image']('Add'))
            self.Add_Image.Bind(On_Click=lambda E: self.Add())
            self.Add_Image.Create()
            
                #Delete Image
            Fixture = self.Frame.Locate(3.5, 5, 91.5, 6.5)
            self.Delete_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Image.Config(Background=False, Border_Size=0, Hand_Cursor=True)
            self.Delete_Image.Config(Path=self.Global['Image']('Delete'))
            self.Delete_Image.Bind(On_Click=lambda E: self.Delete_Show())
            self.Delete_Image.Create()
            
                #Delete Confirm Image
            Fixture = self.Frame.Locate(3, 4, 88, 7)
            self.Delete_Confirm_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Confirm_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Confirm_Image.Config(Background=False, Border_Size=0, Hand_Cursor=True)
            self.Delete_Confirm_Image.Config(Path=self.Global['Image']('Success'))
            self.Delete_Confirm_Image.Bind(On_Click=lambda E: self.Delete())
            self.Delete_Confirm_Image.Create()
            
                #Delete Cancel Image
            Fixture = self.Frame.Locate(3, 4, 91.5, 7)
            self.Delete_Cancel_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Cancel_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Cancel_Image.Config(Background=False, Border_Size=0, Hand_Cursor=True)
            self.Delete_Cancel_Image.Config(Path=self.Global['Image']('Close'))
            self.Delete_Cancel_Image.Bind(On_Click=lambda E: self.Delete_Hide())
            self.Delete_Cancel_Image.Create()
            
                #Title Label
            Fixture = self.Frame.Locate(7, 5, 3, 16)
            self.Title_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Title_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Title_Label.Config(Foreground='black', Value="Title:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Title_Label.Create()
            
                #Title Entry
            Fixture = self.Frame.Locate(30, 5, 12, 16)
            self.Title_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Title_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Title_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='left', Border_Size=1)
            self.Title_Entry.Create()
            
                #Background Label
            Fixture = self.Frame.Locate(15, 5, 47, 16)
            self.Background_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Background_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Background_Label.Config(Foreground='black', Value="Background:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Background_Label.Create()
            
                #Background Color
            Fixture = self.Frame.Locate(4, 5, 61, 16)
            self.Background_Color = self.Global['Gluonix'].Label(self.Frame)
            self.Background_Color.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Background_Color.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='w', Border_Size=1)
            self.Background_Color.Bind(On_Click=lambda E: self.Select_Color())
            self.Background_Color.Create()
            
                #Alignment Label
            Fixture = self.Frame.Locate(12, 5, 70, 16)
            self.Alignment_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Alignment_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Alignment_Label.Config(Foreground='black', Value="Alignment:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Alignment_Label.Create()
            
                #Alignment Select
            Fixture = self.Frame.Locate(12, 5, 82, 16)
            self.Alignment_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Alignment_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Alignment_Select.Config(Background='white', Foreground='black', Font_Size=11, Font_Weight='normal', Align='center', Border_Size=1)
            self.Alignment_Select.Create()
            self.Alignment_Select.Add('Percentage')
            self.Alignment_Select.Add('Pixel')
            
                #Width Label
            Fixture = self.Frame.Locate(7, 5, 3, 25)
            self.Width_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Width_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Width_Label.Config(Foreground='black', Value="Width:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Width_Label.Create()

                #Width Entry
            Fixture = self.Frame.Locate(7, 5, 12, 25)
            self.Width_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Width_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Width_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='center', Border_Size=1)
            self.Width_Entry.Create()
            
                #Height Label
            Fixture = self.Frame.Locate(7, 5, 22, 25)
            self.Height_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Height_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Height_Label.Config(Foreground='black', Value="Height:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Height_Label.Create()

                #Width Entry
            Fixture = self.Frame.Locate(7, 5, 31, 25)
            self.Height_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Height_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Height_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='center', Border_Size=1)
            self.Height_Entry.Create()
            
                #Left Label
            Fixture = self.Frame.Locate(7, 5, 41, 25)
            self.Left_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Left_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Left_Label.Config(Foreground='black', Value="Left:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Left_Label.Create()

                #Left Entry
            Fixture = self.Frame.Locate(7, 5, 49, 25)
            self.Left_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Left_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Left_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='center', Border_Size=1)
            self.Left_Entry.Create()
            
                #Top Label
            Fixture = self.Frame.Locate(7, 5, 59, 25)
            self.Top_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Top_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Top_Label.Config(Foreground='black', Value="Top:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Top_Label.Create()

                #Top Entry
            Fixture = self.Frame.Locate(7, 5, 67, 25)
            self.Top_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Top_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Top_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='center', Border_Size=1)
            self.Top_Entry.Create()
            
                #Full Screen Label
            Fixture = self.Frame.Locate(12, 5, 3, 34)
            self.Full_Screen_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Full_Screen_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Full_Screen_Label.Config(Foreground='black', Value="Full Screen:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Full_Screen_Label.Create()
            
                #Full Screen Check
            Fixture = self.Frame.Locate(4, 5, 15, 34)
            self.Full_Screen_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Full_Screen_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Full_Screen_Check.Config(Border_Size=0)
            self.Full_Screen_Check.Create()
            
                #Resizable Label
            Fixture = self.Frame.Locate(12, 5, 26, 34)
            self.Resizable_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Resizable_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resizable_Label.Config(Foreground='black', Value="Resizable:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Resizable_Label.Create()
            
                #Resizable Check
            Fixture = self.Frame.Locate(4, 5, 38, 34)
            self.Resizable_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Resizable_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resizable_Check.Config(Border_Size=0)
            self.Resizable_Check.Create()
            
                #Persistent Label
            Fixture = self.Frame.Locate(12, 5, 49, 34)
            self.Persistent_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Persistent_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Persistent_Label.Config(Foreground='black', Value="Persistent:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Persistent_Label.Create()
            
                #Persistent Check
            Fixture = self.Frame.Locate(4, 5, 61, 34)
            self.Persistent_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Persistent_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Persistent_Check.Config(Border_Size=0)
            self.Persistent_Check.Create()
            
                #Menu Label
            Fixture = self.Frame.Locate(12, 5, 72, 34)
            self.Menu_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Menu_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Menu_Label.Config(Foreground='black', Value="Menu:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Menu_Label.Create()
            
                #Menu Check
            Fixture = self.Frame.Locate(4, 5, 84, 34)
            self.Menu_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Menu_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Menu_Check.Config(Border_Size=0)
            self.Menu_Check.Create()
            
                #Toolbar Label
            Fixture = self.Frame.Locate(12, 5, 3, 43)
            self.Toolbar_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Toolbar_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Toolbar_Label.Config(Foreground='black', Value="Toolbar:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Toolbar_Label.Create()
            
                #Toolbar Check
            Fixture = self.Frame.Locate(4, 5, 15, 43)
            self.Toolbar_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Toolbar_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Toolbar_Check.Config(Border_Size=0)
            self.Toolbar_Check.Create()
            
                #Error Log Label
            Fixture = self.Frame.Locate(12, 5, 26, 43)
            self.Error_Log_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Error_Log_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Error_Log_Label.Config(Foreground='black', Value="Error Log:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Error_Log_Label.Create()
            
                #Error Log Check
            Fixture = self.Frame.Locate(4, 5, 38, 43)
            self.Error_Log_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Error_Log_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Error_Log_Check.Config(Border_Size=0)
            self.Error_Log_Check.Create()
            
                #Topmost Label
            Fixture = self.Frame.Locate(12, 5, 26, 43)
            self.Topmost_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Topmost_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Topmost_Label.Config(Foreground='black', Value="Topmost:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Topmost_Label.Create()
            self.Topmost_Label.Hide()
            
                #Topmost Check
            Fixture = self.Frame.Locate(4, 5, 38, 43)
            self.Topmost_Check = self.Global['Gluonix'].Check(self.Frame)
            self.Topmost_Check.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Topmost_Check.Config(Border_Size=0)
            self.Topmost_Check.Create()
            self.Topmost_Check.Hide()
            
                #Version Label
            Fixture = self.Frame.Locate(12, 5, 49, 43)
            self.Version_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Version_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Version_Label.Config(Foreground='black', Value="Version:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Version_Label.Create()
            
                #Version Entry
            Fixture = self.Frame.Locate(6, 5, 61, 43)
            self.Version_Entry = self.Global['Gluonix'].Spinner(self.Frame)
            self.Version_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Version_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='center', Border_Size=1)
            self.Version_Entry.Create()
            
                #Revision Label
            Fixture = self.Frame.Locate(12, 5, 72, 43)
            self.Revision_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Revision_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Revision_Label.Config(Foreground='black', Value="Revision:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Revision_Label.Create()
            
                #Revision Entry
            Fixture = self.Frame.Locate(6, 5, 84, 43)
            self.Revision_Entry = self.Global['Gluonix'].Spinner(self.Frame)
            self.Revision_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Revision_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='center', Border_Size=1)
            self.Revision_Entry.Create()
            
                #Icon Label
            Fixture = self.Frame.Locate(7, 5, 3, 52)
            self.Icon_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Icon_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Icon_Label.Config(Foreground='black', Value="Icon:", Font_Size=12, Font_Weight='normal', Align='w', Border_Size=0)
            self.Icon_Label.Create()
            
                #Icon Image
            Fixture = self.Frame.Locate(15, 15, 12, 52)
            self.Icon_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Icon_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Icon_Image.Config(Background='white', Border_Size=1, Hand_Cursor=True)
            self.Icon_Image.Create()
            
                #Icon Browser
            Fixture = self.Frame.Locate(12, 6, 30, 52)
            self.Icon_Browser = self.Global['Gluonix'].Label(self.Frame)
            self.Icon_Browser.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Icon_Browser.Config(Background='#e1e1e1', Foreground='black', Value='Browser...', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Icon_Browser.Bind(On_Hover_In=lambda E: self.Icon_Browser.Config(Border_Color='#0078d7', Background='#d5dcf0'))
            self.Icon_Browser.Bind(On_Hover_Out=lambda E: self.Icon_Browser.Config(Border_Color='#adadad', Background='#e1e1e1'))
            self.Icon_Browser.Bind(On_Click=lambda E: self.Select_Icon())
            self.Icon_Browser.Create()
            
                #Deploy Button
            Fixture = self.Frame.Locate(8, 6, 66, 88)
            self.Deploy_Button = self.Global['Gluonix'].Label(self.Frame)
            self.Deploy_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Deploy_Button.Config(Background='#D2B4DE', Foreground='black', Value='Deploy', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#5B2C6F')
            self.Deploy_Button.Bind(On_Hover_In=lambda E: self.Deploy_Button.Config(Background='#8E44AD'))
            self.Deploy_Button.Bind(On_Hover_Out=lambda E: self.Deploy_Button.Config(Background='#D2B4DE'))
            self.Deploy_Button.Bind(On_Click=lambda E: self.Deploy())
            self.Deploy_Button.Create()
            
                #Design Button
            Fixture = self.Frame.Locate(8, 6, 76, 88)
            self.Design_Button = self.Global['Gluonix'].Label(self.Frame)
            self.Design_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Design_Button.Config(Background='#FAD7A0', Foreground='black', Value='Design', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#935116')
            self.Design_Button.Bind(On_Hover_In=lambda E: self.Design_Button.Config(Background='#F39C12'))
            self.Design_Button.Bind(On_Hover_Out=lambda E: self.Design_Button.Config(Background='#FAD7A0'))
            self.Design_Button.Bind(On_Click=lambda E: self.Save_Check(Launch=True, Direct=True))
            self.Design_Button.Create()
            
                #Save Button
            Fixture = self.Frame.Locate(8, 6, 86, 88)
            self.Save_Button = self.Global['Gluonix'].Label(self.Frame)
            self.Save_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Save_Button.Config(Background='#ABEBC6', Foreground='black', Value='Save', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#1D8348')
            self.Save_Button.Bind(On_Hover_In=lambda E: self.Save_Button.Config(Background='#2ECC71'))
            self.Save_Button.Bind(On_Hover_Out=lambda E: self.Save_Button.Config(Background='#ABEBC6'))
            self.Save_Button.Bind(On_Click=lambda E: self.Save_Check())
            self.Save_Button.Create()
            
                #Resize Frame
            Fixture = self.Frame.Locate(28, 20, 66, 66)
            self.Resize_Frame = self.Global['Gluonix'].Frame(self.Frame)
            self.Resize_Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Frame.Config(Border_Size=3, Display=False, Background='#FFFFFF')
            self.Resize_Frame.Create()
            
                    #Resize Label
            Fixture = self.Resize_Frame.Locate(90, 50, 5, 10)
            self.Resize_Label = self.Global['Gluonix'].Label(self.Resize_Frame)
            self.Resize_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Label.Config(Foreground='black', Value="", Font_Size=12, Font_Weight='normal', Align='center', Border_Size=0)
            self.Resize_Label.Create()
            self.Resize_Label.Set("Resize all widgets to adjust for new screen size?")
            
                    #Resize Reject Button
            Fixture = self.Resize_Frame.Locate(35, 25, 55, 65)
            self.Resize_Reject_Button = self.Global['Gluonix'].Label(self.Resize_Frame)
            self.Resize_Reject_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Reject_Button.Config(Background='#F5B7B1', Foreground='black', Value='Reject', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#1D8348')
            self.Resize_Reject_Button.Bind(On_Hover_In=lambda E: self.Resize_Reject_Button.Config(Background='#E74C3C'))
            self.Resize_Reject_Button.Bind(On_Hover_Out=lambda E: self.Resize_Reject_Button.Config(Background='#F5B7B1'))
            self.Resize_Reject_Button.Bind(On_Click=lambda E: self.Resize_Reject())
            self.Resize_Reject_Button.Create()
            
                    #Resize Accept Button
            Fixture = self.Resize_Frame.Locate(35, 25, 10, 65)
            self.Resize_Accept_Button = self.Global['Gluonix'].Label(self.Resize_Frame)
            self.Resize_Accept_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Resize_Accept_Button.Config(Background='#ABEBC6', Foreground='black', Value='Accept', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#1D8348')
            self.Resize_Accept_Button.Bind(On_Hover_In=lambda E: self.Resize_Accept_Button.Config(Background='#2ECC71'))
            self.Resize_Accept_Button.Bind(On_Hover_Out=lambda E: self.Resize_Accept_Button.Config(Background='#ABEBC6'))
            self.Resize_Accept_Button.Bind(On_Click=lambda E: self.Resize_Accept())
            self.Resize_Accept_Button.Create()
            
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Save(self, Loading=True):
        try:
            if Loading:
                self.Global['Loading'].Show()
                self.Global['GUI'].After(500, lambda:self.Save(Loading=False))
            else:
                Error = False
                if not Error and self.Title_Entry.Get()=='':
                    Error = True
                    self.Global['Message'].Show('Error', 'Enter Project Title')
                if not Error and not self.Width_Entry.Get().isdigit():
                    Error = True
                    self.Global['Message'].Show('Error', 'Incorrect Width Value')
                if not Error and not self.Height_Entry.Get().isdigit():
                    Error = True
                    self.Global['Message'].Show('Error', 'Incorrect Height Value')
                if not Error and not self.Left_Entry.Get().isdigit():
                    Error = True
                    self.Global['Message'].Show('Error', 'Incorrect Left Value')
                if not Error and not self.Top_Entry.Get().isdigit():
                    Error = True
                    self.Global['Message'].Show('Error', 'Incorrect Top Value')
                if not Error and not self.Version_Entry.Get().isdigit():
                    Error = True
                    self.Global['Message'].Show('Error', 'Incorrect Version Value')
                if not Error and not self.Revision_Entry.Get().isdigit():
                    Error = True
                    self.Global['Message'].Show('Error', 'Incorrect Revision Value')
                if not Error:
                    self.Display_ID = self.Display_Select.Get()
                    Project_Name = self.Title_Entry.Get()
                    if self.Display_ID=='Root':
                        for File_Name in os.listdir(self.Project_Path):
                            if File_Name.endswith('.ng'):
                                os.remove(f'{self.Project_Path}/{File_Name}')
                        with open(f'{self.Project_Path}/{Project_Name}.ng', 'w') as File:
                            File.write(Project_Name)
                        self.Project_Data['Name'] = Project_Name
                    self.Project_Data['Error_Log'] = str(int(self.Error_Log_Check.Get()))
                    self.Project_Data['Version'] = self.Version_Entry.Get()
                    self.Project_Data['Revision'] = self.Revision_Entry.Get()
                    self.Project_Data['NGD'] = f"{self.Global['Version']}.{self.Global['Revision']}"
                    self.Display_Data['Title'] = Project_Name
                    self.Display_Data['Background'] = self.Background_Color.Config_Get('Background')['Background']
                    self.Display_Data['Alignment'] = self.Alignment_Select.Get()
                    self.Display_Data['Width'] = self.Width_Entry.Get()
                    self.Display_Data['Height'] = self.Height_Entry.Get()
                    self.Display_Data['Left'] = self.Left_Entry.Get()
                    self.Display_Data['Top'] = self.Top_Entry.Get()
                    self.Display_Data['Full_Screen'] = str(int(self.Full_Screen_Check.Get()))
                    self.Display_Data['Resizable'] = str(int(self.Resizable_Check.Get()))
                    self.Display_Data['Persistent'] = str(int(self.Persistent_Check.Get()))
                    self.Display_Data['Menu'] = str(int(self.Menu_Check.Get()))
                    self.Display_Data['Toolbar'] = str(int(self.Toolbar_Check.Get()))
                    self.Display_Data['Topmost'] = str(int(self.Topmost_Check.Get()))
                    TempDatabase = self.Global['Gluonix'].SQL(f'{self.Project_Path}/Data/NGD.dll')
                    Display_Data_Temp = TempDatabase.Get(f"SELECT * FROM `Display` WHERE `ID`='{self.Display_ID}'", Keys=True)
                    TempDatabase.Post(f"UPDATE `Display` SET `Title`='{self.Display_Data['Title']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Background`='{self.Display_Data['Background']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Alignment`='{self.Display_Data['Alignment']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Width`='{self.Display_Data['Width']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Height`='{self.Display_Data['Height']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Left`='{self.Display_Data['Left']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Top`='{self.Display_Data['Top']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Full_Screen`='{self.Display_Data['Full_Screen']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Resizable`='{self.Display_Data['Resizable']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Persistent`='{self.Display_Data['Persistent']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Menu`='{self.Display_Data['Menu']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Toolbar`='{self.Display_Data['Toolbar']}' WHERE `ID`='{self.Display_ID}'")
                    TempDatabase.Post(f"UPDATE `Display` SET `Topmost`='{self.Display_Data['Topmost']}' WHERE `ID`='{self.Display_ID}'")
                    if f"{self.Project_Path}/Data/File/{self.Project_Data['Icon']}"!=self.Icon_Image.Config_Get('Path')['Path']:
                        if os.path.exists(f"{self.Project_Path}/Data/File/{self.Project_Data['Icon']}"):
                            os.remove(f"{self.Project_Path}/Data/File/{self.Project_Data['Icon']}")
                        shutil.copy(self.Icon_Image.Config_Get('Path')['Path'], f"{self.Project_Path}/Data/File/{self.Project_Data['Icon']}")
                    if not self.Direct:
                        self.Direct = False
                        self.Global['Loading'].Hide()
                        self.Global['Message'].Show('Success', 'Save Successfull')
                        self.Global['Message'].Hide(Delay=2)
                    if self.Resize:
                        self.Resize = False
                        self.Display_Data = Display_Data_Temp[0]
                        Old_Width = self.Display_Data['Width']
                        Old_Height = self.Display_Data['Height']
                        Current_Width = int(self.Width_Entry.Get())
                        Current_Height = int(self.Height_Entry.Get())
                        Width_Ratio = float(Current_Width) / Old_Width
                        Height_Ratio = float(Current_Height) / Old_Height
                        if Width_Ratio < 1.0 or Height_Ratio < 1.0:
                            Font_Ratio = min(Width_Ratio, Height_Ratio)
                        else:
                            Font_Ratio = (Width_Ratio * Height_Ratio) ** 0.5
                        def Update_Size(Frame_ID):
                            TempDatabase.Post(f"UPDATE `Frame` SET Width=Width*{Width_Ratio}, Height=Height*{Height_Ratio}, Left=Left*{Width_Ratio}, Top=Top*{Height_Ratio} WHERE `ID`='{Frame_ID}'")
                            TempDatabase.Post(f"UPDATE `Widget` SET Width=Width*{Width_Ratio}, Height=Height*{Height_Ratio}, Left=Left*{Width_Ratio}, Top=Top*{Height_Ratio}, Font_Size=CAST(Font_Size*{Font_Ratio} AS INTEGER) WHERE `Root`='{Frame_ID}'")
                            TempDatabase.Post(f"UPDATE `Item` SET Width=Width*{Width_Ratio}, Height=Height*{Height_Ratio}, Left=Left*{Width_Ratio}, Top=Top*{Height_Ratio}, Size=CAST(Size*{Font_Ratio} AS INTEGER) WHERE `Root`='{Frame_ID}'")
                            Frames = TempDatabase.Get(f"SELECT * FROM `Frame` WHERE `Root`='{Frame_ID}'", Keys=True)
                            for Frame in Frames:
                                Update_Size(Frame['ID'])
                        Update_Size(self.Display_ID)
                    if self.Launch:
                        self.Launch = False
                        self.Panel.Home.Main.Design.Configure.Reset_All()
                        self.Panel.Home.Main.Design.Project_Path = self.Project_Path
                        self.Panel.Home.Main.Design.Project_Data = self.Project_Data
                        self.Panel.Home.Main.Design.Display_ID = self.Display_ID
                        self.Panel.Home.Frame.Hide()
                        self.Panel.Home.Main.Design.Update(Loading=False)
                    TempDatabase.Close()
                    return True
                else:
                    self.Global['Loading'].Hide()
                    return False
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Save_Check(self, Launch=False, Direct=False):
        try:
            self.Launch = Launch
            self.Direct = Direct
            Save = True
            TempDatabase = self.Global['Gluonix'].SQL(f'{self.Project_Path}/Data/NGD.dll')
            Display_Data_Temp = TempDatabase.Get(f"SELECT * FROM `Display` WHERE `ID`='{self.Display_ID}'", Keys=True)
            self.Display_Data = Display_Data_Temp[0]
            Old_Alignment = self.Display_Data['Alignment']
            Old_Width = self.Display_Data['Width']
            Old_Height = self.Display_Data['Height']
            Current_Alignment = self.Alignment_Select.Get()
            Current_Width = int(self.Width_Entry.Get())
            Current_Height = int(self.Height_Entry.Get())
            if Old_Alignment=='Pixel' and Current_Alignment=='Pixel':
                if Old_Width!=Current_Width or Old_Height!=Current_Height:
                    Save = False
                    self.Resize_Frame.Show()
            if Save:
                self.Save()
            return True
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Resize_Accept(self):
        try:
            self.Resize_Frame.Hide()
            self.Resize = True
            self.Save()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Resize_Reject(self):
        try:
            self.Resize_Frame.Hide()
            self.Resize = False
            self.Save()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Deploy(self, Loading=True):
        try:
            if Loading:
                if self.Runtime_Path:
                    Initial = self.Runtime_Path
                else:
                    Initial = os.path.join(os.path.expanduser('~'), 'Documents')
                Path = self.Global['GUI'].Folder(Initial=Initial, Title='Select Runtime Folder')
                if Path:
                    Temp_Path = Path.replace('/', '\\')
                    if Temp_Path!=self.Project_Path:
                        self.Runtime_Path = Path
                        self.Global['Loading'].Show()
                        self.Global['GUI'].After(500, lambda:self.Deploy(Loading=False))
                    else:
                        self.Global['Message'].Show('Error', 'Project Path Cannot Be Runtime Path')
            else:
                if self.Save_Check(Direct=True):
                    if os.path.exists(f'{self.Runtime_Path}/Nucleon'):
                        shutil.rmtree(f'{self.Runtime_Path}/Nucleon')
                    shutil.copytree(self.Global['Relative_Path']('Nucleon'), f'{self.Runtime_Path}/Nucleon')
                    shutil.copytree(f'{self.Project_Path}/Data', f'{self.Runtime_Path}/Nucleon/Data')
                    Temp_Name = self.Title_Entry.Get()
                    if not os.path.exists(f'{self.Runtime_Path}/{Temp_Name}.py'):
                        shutil.copy(self.Global['Relative_Path']('Program/Base/GUI.py'), f'{self.Runtime_Path}/{Temp_Name}.py')
                    self.Global['Message'].Show('Success', 'Project Deployed')
                    if self.Global['GUI']._Window:
                        os.startfile(self.Runtime_Path)
                self.Global['Loading'].Hide()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Add(self):
        try:
            TempDatabase = self.Global['Gluonix'].SQL(f'{self.Project_Path}/Data/NGD.dll')
            X = 1
            Popup_Data = TempDatabase.Get(f"SELECT * FROM `Display` WHERE `ID`='Popup{X}'", Keys=True)
            while len(Popup_Data)>0:
                X+=1
                Popup_Data = TempDatabase.Get(f"SELECT * FROM `Display` WHERE `ID`='Popup{X}'", Keys=True)
            ID = f'Popup{X}'
            TempDatabase.Post(f"INSERT INTO `Display` (`ID`) VALUES ('{ID}')")
            self.Display_Select.Add(ID)
            self.Display_Select.Set(ID)
            self.Update(Partial=True)
            TempDatabase.Close()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete_Show(self):
        try:
            self.Delete_Image.Hide()
            self.Delete_Confirm_Image.Show()
            self.Delete_Cancel_Image.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete_Hide(self):
        try:
            self.Delete_Confirm_Image.Hide()
            self.Delete_Cancel_Image.Hide()
            if self.Display_Select.Get()!='Root':
                self.Delete_Image.Show()
            else:
                self.Delete_Image.Hide()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete(self):
        try:
            self.TempDatabase = self.Global['Gluonix'].SQL(f'{self.Project_Path}/Data/NGD.dll')
            Display_ID = self.Display_Select.Get()
            if Display_ID==self.Display_ID and Display_ID!='Root' and self.Display_ID!='Root':
                self.Delete_All(Display_ID)
            self.TempDatabase.Close()
            self.TempDatabase = False
            self.Display_Select.Remove(Display_ID)
            self.Display_Select.Set('Root')
            self.Update(Partial=True)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete_All(self, Parent=False):
        try:
            if Parent:
                Frames = self.TempDatabase.Get(f"SELECT * FROM `Frame` WHERE `Root`='{Parent}'", Keys=True)
                for Frame in Frames:
                    self.Delete_All(Parent=Frame['ID'])
                self.TempDatabase.Post(f"Delete FROM `Widget` WHERE `Root`='{Parent}'")
                self.TempDatabase.Post(f"Delete FROM `Frame` WHERE `ID`='{Parent}'")
                self.TempDatabase.Post(f"Delete FROM `Display` WHERE `ID`='{Parent}'")
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update(self, Partial=False):
        try:
            if os.path.exists(f'{self.Project_Path}/Data/NGD.dll'):
                TempDatabase = self.Global['Gluonix'].SQL(f'{self.Project_Path}/Data/NGD.dll')
                if not Partial:
                    self.Project_Data = {}
                    Project_Data_Temp = TempDatabase.Get("SELECT * FROM `Variable`", Keys=True)
                    for Each in Project_Data_Temp:
                        self.Project_Data[Each['ID']] = Each['Data']
                    Display_Data_Temp = TempDatabase.Get("SELECT * FROM `Display`", Keys=True)
                    self.Display_Select.Clear()
                    for Each in Display_Data_Temp:
                        self.Display_Select.Add(Each['ID'])
                    self.Display_Select.Set('Root')
                    self.Error_Log_Check.Set(bool(int(self.Project_Data['Error_Log'])))
                    self.Version_Entry.Set(self.Project_Data['Version'])
                    self.Revision_Entry.Set(self.Project_Data['Revision'])
                    self.Icon_Image.Set(f"{self.Project_Path}/Data/File/{self.Project_Data['Icon']}")
                self.Display_ID = self.Display_Select.Get()
                self.Display_Select.Sort()
                Display_Data_Temp = TempDatabase.Get(f"SELECT * FROM `Display` WHERE `ID`='{self.Display_ID}'", Keys=True)
                self.Display_Data = Display_Data_Temp[0]
                self.Title_Entry.Set(self.Display_Data['Title'])
                self.Background_Color.Config(Background=self.Display_Data['Background'])
                self.Alignment_Select.Set(self.Display_Data['Alignment'])
                self.Width_Entry.Set(self.Display_Data['Width'])
                self.Height_Entry.Set(self.Display_Data['Height'])
                self.Left_Entry.Set(self.Display_Data['Left'])
                self.Top_Entry.Set(self.Display_Data['Top'])
                self.Full_Screen_Check.Set(bool(int(self.Display_Data['Full_Screen'])))
                self.Resizable_Check.Set(bool(int(self.Display_Data['Resizable'])))
                self.Persistent_Check.Set(bool(int(self.Display_Data['Persistent'])))
                self.Menu_Check.Set(bool(int(self.Display_Data['Menu'])))
                self.Toolbar_Check.Set(bool(int(self.Display_Data['Toolbar'])))
                self.Topmost_Check.Set(bool(int(self.Display_Data['Topmost'])))
                if self.Display_ID=='Root':
                    self.Error_Log_Label.Show()
                    self.Error_Log_Check.Show()
                    self.Topmost_Label.Hide()
                    self.Topmost_Check.Hide()
                    self.Version_Label.Show()
                    self.Version_Entry.Show()
                    self.Revision_Label.Show()
                    self.Revision_Entry.Show()
                    self.Icon_Label.Show()
                    self.Icon_Image.Show()
                    self.Icon_Browser.Show()
                else:
                    self.Topmost_Label.Show()
                    self.Topmost_Check.Show()
                    self.Error_Log_Label.Hide()
                    self.Error_Log_Check.Hide()
                    self.Version_Label.Hide()
                    self.Version_Entry.Hide()
                    self.Revision_Label.Hide()
                    self.Revision_Entry.Hide()
                    self.Icon_Label.Hide()
                    self.Icon_Image.Hide()
                    self.Icon_Browser.Hide()
                self.Delete_Hide()
                self.Frame.Show()
                TempDatabase.Close()
            else:
                self.Global['Message'].Show('Error', 'Project files not found')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Select_Color(self):
        try:
            Color = colorchooser.askcolor(color=self.Background_Color.Config_Get('Background')['Background'], title ="Choose Background Color")[1]
            if Color is not None:
                self.Background_Color.Config(Background=Color)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
    
    def Select_Icon(self):
        try:
            Icon_File_Path = self.Global['GUI'].File(Initial=os.path.join(os.path.expanduser('~'), 'Documents'), Title='Select Icon', Default='.ico', Type=[["Icon (*.ico)", "*.ico"]])
            if Icon_File_Path:
                self.Icon_Image.Set(Icon_File_Path)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


# IMPORT LIBRARIES
import math
from .N_Custom import Event_Bind_Canvas
        
class Canvas_Pie:
    def __init__(self, Main):
        self._Canvas = Main
        self._Config = ['Name', 'Outline', 'Fill', 'Left', 'Top', 'Radius', 'Thickness', 'Resize', 'Start', 'Extent', 'Translucent']
        self._Display = True
        self._Resize = True
        self._Name = False
        self._Last_Name = False
        self._Type = 'Canvas_Pie'
        self._Outline = '#000000'
        self._Fill = '#000000'
        self._Thickness = 1
        self._Translucent = False
        self._Start = 0
        self._Extent = 180
        self._Left, self._Top, self._Radius = 0, 0, 0
        self._X1 = abs(self._Left - self._Radius)
        self._Y1 = abs(self._Top - self._Radius)
        self._X2 = abs(self._Left + self._Radius)
        self._Y2 = abs(self._Top + self._Radius)
        self._Widget = self._Canvas._Frame.create_arc(self._X1, self._Y1, self._X2, self._Y2, start=self._Start, extent=self._Extent, outline=self._Outline, width=self._Thickness, fill=self._Fill)
        self._Canvas._Widget.append(self)
        self._Resizable = self._Canvas._Resizable
        self._On_Show = False
        self._On_Hide = False

    def Copy(self, Name=False, Main=False):
        try:
            if not Main:
                Main = self._Canvas
            Instance = type(self)(Main)
            for Key in self._Config:
                setattr(Instance, "_"+Key, getattr(self, "_"+Key))
            if Name:
                setattr(Instance, "_Name", Name)
            Instance.Create()
            return Instance
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Copy -> {E}")

    def Hide(self):
        try:
            self._Canvas._Frame.itemconfigure(self._Widget, state='hidden')
            self._Display = False
            if self._On_Hide:
                self._On_Hide()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type}-> Hide -> {E}")
            
    def Show(self):
        try:
            self._Display = True
            if self._Resizable:
                self.Resize()
            else:
                self.Display()
            if self._On_Show:
                self._On_Show()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Show -> {E}")

    def Display(self):
        try:
            self._Canvas._Frame.itemconfigure(self._Widget, state='normal')
            self._Canvas._Frame.tag_raise(self._Widget)
            self._Display = True
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Display -> {E}")
    
    def Delete(self):
        try:
            self._Canvas._Widget.remove(self)
            self._Canvas._Frame.delete(self._Widget)
            if self:
                del self
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Delete -> {E}")
        
    def Bind(self, **Input):
        try:
            if 'On_Show' in Input:
                self._On_Show = Input['On_Show']
            if 'On_Hide' in Input:
                self._On_Hide = Input['On_Hide']
            Event_Bind_Canvas(self._Canvas._Frame, self._Widget, **Input)
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Bind -> {E}")
            
    def Config_Get(self, *Input):
        try:
            Return = {}
            for Each in self._Config:
                if Each in Input:
                    Return[Each] = getattr(self, "_"+Each)
            return Return
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Config_Get -> {E}")
                
    def Config(self, **Input):
        try:
            Run = False
            for Each in self._Config:
                if Each in Input:
                    Value = Input[Each]
                    setattr(self, "_"+Each, Value)
                    setattr(self, "_"+Each+"_Current", Value)
                    Run = True
            if Run:
                self.Relocate()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Config -> {E}")
        
    def Position(self, Left=None, Top=None):
        try:
            if Left is not None:
                self._Left = Left
            if Top is not None:
                self._Top = Top
            if Left is not None or Top is not None:
                self.Relocate()
            Box = self._Canvas._Frame.bbox(self._Widget)
            X1, Y1, X2, Y2 = Box
            return [X1, Y1]
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Position -> {E}")
            
    def Size(self, Width=False, Height=False):
        try:
            if Width:
                self._Width = Width
            if Height:
                self._Height = Height
            if Width or Height:
                self.Relocate()
            Box = self._Canvas._Frame.bbox(self._Widget)
            X1, Y1, X2, Y2 = Box
            Width = X2 - X1
            Height = Y2 - Y1
            return [Width, Height]
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Size -> {E}")
        
    def Create(self):
        try:
            Stripple = 'gray12' if self._Translucent else ''
            self._Canvas._Frame.itemconfig(self._Widget, start=self._Start, extent=self._Extent, outline=self._Outline, width=self._Thickness, fill=self._Fill, stipple=Stripple)
            self._Canvas._Frame.coords(self._Widget, self._X1, self._Y1, self._X2, self._Y2)
            if self._Name!=self._Last_Name:
                if self._Last_Name:
                    if self._Last_Name in self._Canvas.__dict__:
                        del self._Canvas.__dict__[self._Last_Name]
                if self._Name:
                    self._Canvas.__dict__[self._Name] = self
                self._Last_Name = self._Name
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Create -> {E}")

    def Adjustment(self):
        try:
            self._Width_Ratio = self._Canvas._Width_Current / self._Canvas._Width
            self._Height_Ratio = self._Canvas._Height_Current / self._Canvas._Height
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Adjustment -> {E}")
            
    def Relocate(self, Direct=False):
        try:
            if self._Resize:
                self.Adjustment()
                Left = self._Left * self._Width_Ratio
                Top  = self._Top * self._Height_Ratio
                Ratio = min(self._Width_Ratio, self._Height_Ratio)
                Radius = self._Radius * Ratio
                self._X1 = abs(Left - Radius)
                self._Y1 = abs(Top - Radius)
                self._X2 = abs(Left + Radius)
                self._Y2 = abs(Top + Radius)
            self.Create()
            if self._Display:
                self.Display()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Relocate -> {E}")
            
    def Resize(self):
        try:
            self.Relocate()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Resize -> {E}")


################################################################################################################################
#Stock_Line
################################################################################################################################
import inspect
import time
import random
import string

class Stock_Line:
    def __init__(self, Global, Stock, Left, Top):
        try:
            self.Global = Global
            self.Stock = Stock
            self.Widget= []
            self.Type = 'Line'
            
            #Label
            Fixture = self.Stock.Scroll.Locate(30, 5, Left, Top)
            self.Label = self.Global['Gluonix'].Compound(self.Stock.Scroll)
            self.Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label.Config(Background='#FFFFFF', Border_Size=1, Border_Color='#adadad', Display=True)
            self.Label.Config(Foreground='#000000', Font_Size=11, Font_Weight='normal', Align='center', Compound='left')
            self.Label.Config(Resize=True, Move=True)
            self.Label.Bind(On_Hover_In=lambda E: self.Label.Config(Border_Color='#0078d7', Background='#d5dcf0'))
            self.Label.Bind(On_Hover_Out=lambda E: self.Label.Config(Border_Color='#adadad', Background='#FFFFFF'))
            self.Label.Bind(On_Click=lambda E: self.On_Click())
            self.Label.Create()
            self.Label.Set(Path=self.Global['Image'](self.Type), Value=' Line')
            self.Stock.Widget.append(self.Label)
            
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def On_Click(self):
        try:
            ID_List = self.Stock.Design.Element.Tree.Get()
            if not ID_List:
                Root_ID = self.Stock.Design.Element.Parent
                Root_Type = 'Frame'
                Root_Level = 0
            else:
                Root_ID = ID_List[0]
                Root_Type = ID_List[1]
                Root_Level = ID_List[2]
            Level = Root_Level+1
            if (Root_Type!='Frame' and Root_Type!='Canvas' and Root_Type!='Scroll'):
                Widget = self.Stock.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `ID`='{Root_ID}'", Keys=True)
                if len(Widget)==0:
                    Widget = self.Stock.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Root_ID}'", Keys=True)
                Root_ID = Widget[0]['Root']
                Level = Root_Level
            Number = 1
            Exist = self.Stock.Design.Database.Get(f"SELECT * FROM `Widget` WHERE (`Name`='{self.Type}{Number}' AND `Root`='{Root_ID}')")
            while len(Exist)>0:
                Number += 1
                Exist = self.Stock.Design.Database.Get(f"SELECT * FROM `Widget` WHERE (`Name`='{self.Type}{Number}' AND `Root`='{Root_ID}')")
            Name = f'{self.Type}{Number}'
            Random_Letter = ''.join(random.choices(string.ascii_letters, k=10))
            ID = Random_Letter+self.Global['Custom'].MD5(Root_ID+Name+str(time.time()*1000000))
            Root = getattr(self.Stock.Design.Element, Root_ID)
            Image = self.Global['Image'](self.Type)
            setattr(self.Stock.Design.Element, ID, self.Stock.Design.Element.Tree.Add(Name=f' {Name}', Parent=Root, Value=[ID, self.Type, Level], Path=Image))
            self.Stock.Design.Element.Tree.Expand(Root)
            ID_Tree = getattr(self.Stock.Design.Element, ID)
            self.Stock.Design.Element.Tree.Select(ID_Tree)
            self.Stock.Design.Database.Post(f"INSERT INTO `Widget` (`ID`, `Name`, `Type`, `Root`, `Alignment`) VALUES ('{ID}', '{Name}', '{self.Type}', '{Root_ID}', '{self.Stock.Design.Alignment}')")
            self.Create(ID)
            self.Stock.Design.Configure.Hide_All()
            Configure = getattr(self.Stock.Design.Configure, f'Configure_{self.Type}')
            Configure.Load(ID)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Create(self, ID=False):
        try:
            if ID:
                Widget_Data = self.Stock.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `ID`='{ID}'", Keys=True)
                if len(Widget_Data)==1:
                    Widget_Data = Widget_Data[0]
                    Temp_Root = Widget_Data['Root']
                    Root = Temp_Root
                    while Temp_Root!=self.Stock.Design.Element.Parent:
                        Widget_Data = self.Stock.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root}'", Keys=True)
                        if len(Widget_Data)==1:
                            Widget_Data = Widget_Data[0]
                            Temp_Root = Widget_Data['Root']
                            Root = Temp_Root+'.'+Root
                        else:
                            break
                    Root = self.Global['Custom'].Get_Attr_Class(self.Stock.Design, Root)
                    setattr(Root, ID, self.Global['Gluonix'].Line(Root))
                    Widget_Data = self.Stock.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `ID`='{ID}'", Keys=True)
                    Widget_Data = Widget_Data[0]
                    if Widget_Data['Alignment']=='Percentage':
                        Fixture = Root.Locate(Widget_Data['Width'], Widget_Data['Height'], Widget_Data['Left'], Widget_Data['Top'])
                    else:
                        Fixture = [Widget_Data['Width'], Widget_Data['Height'], Widget_Data['Left'], Widget_Data['Top']]
                    Widget = getattr(Root, ID)
                    Widget._ID = ID
                    Widget.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
                    if Widget_Data['Background']=='False':
                        Widget.Config(Background=False)
                    else:
                        Widget.Config(Background=Widget_Data['Background'])
                    Widget.Config(Foreground=Widget_Data['Foreground'], Border_Size=Widget_Data['Border_Size'], Border_Color=Widget_Data['Border_Color'], Display=True, Disable=bool(Widget_Data['Disabled']))
                    Widget.Config(Font_Size=Widget_Data['Font_Size'], Font_Weight=Widget_Data['Font_Weight'], Align=Widget_Data['Align'], Value=Widget_Data['Value'])
                    Widget.Config(Resize=True, Move=True)
                    Widget.Config(Resize_Width=bool(Widget_Data['Resize_Width']), Resize_Height=bool(Widget_Data['Resize_Height']))
                    Widget.Config(Move_Left=bool(Widget_Data['Move_Left']), Move_Top=bool(Widget_Data['Move_Top']))
                    Widget.Config(Progress=Widget_Data['Progress'], Zero=Widget_Data['Zero'], Increment=Widget_Data['Increment'], Mimimum=Widget_Data['Minimum'], Maximum=Widget_Data['Maximum'])
                    Widget.Config(Path=f"{self.Stock.Design.Project_Path}/Data/File/{ID}")
                    Widget.Config(Url=bool(Widget_Data['Url']), Transparent=bool(Widget_Data['Transparent']), Rotate=Widget_Data['Rotate'], Compound=Widget_Data['Compound'], Aspect_Ratio=bool(Widget_Data['Aspect_Ratio']))
                    Widget.Config(Variable=Widget_Data['Variable'], Secure=Widget_Data['Secure'])
                    Widget.Config(Orient=Widget_Data['Orient'], Ridge=Widget_Data['Ridge'], Height_List=Widget_Data['Height_List'])
                    Widget.Lock = bool(Widget_Data['Lock'])
                    Widget.Create()
                    self.Stock.Design.Element.Intractive(Widget)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


from sqlite3 import connect as SQLite3_connect
from _thread import allocate_lock as Thread_allocate_lock, get_ident as Thread_get_ident

class SQL:
    def __init__(self, Database):
        self.Database = Database
        self.Local = {}
        self.Lock = Thread_allocate_lock()
        self.ReturnData = False
        self.Nothing = False

    def __str__(self):
        return "Sql[Database=" + str(self.Database) + "]"

    def __repr__(self):
        return "Sql[Database=" + str(self.Database) + "]"

    def GetConnection(self):
        ThreadId = Thread_get_ident()
        if ThreadId not in self.Local:
            Connection = SQLite3_connect(self.Database, uri=True, check_same_thread=False)
            Connection.execute("PRAGMA journal_mode=WAL")
            self.Local[ThreadId] = Connection
        return self.Local[ThreadId]

    def Get(self, Command, Keys=False):
        with self.Lock:
            Connection = self.GetConnection()
            Cursor = Connection.cursor()
            Rows = Cursor.execute(Command)
            Rows = Rows.fetchall()
            if Keys and "SELECT *" in Command.upper():
                Columns = Cursor.description
                Column = [Name[0] for Name in Columns]
                self.ReturnData = []
                for Row in Rows:
                    TempRow = {Column[X]: Row[X] for X in range(len(Column))}
                    self.ReturnData.append(TempRow)
            else:
                self.ReturnData = Rows
            return self.ReturnData

    def Post(self, Command, Keys=False):
        with self.Lock:
            Connection = self.GetConnection()
            Cursor = Connection.cursor()
            self.Nothing = Cursor.execute(Command)
            Connection.commit()

    def Close(self):
        ThreadId = Thread_get_ident()
        if ThreadId in self.Local:
            self.Local[ThreadId].close()
            del self.Local[ThreadId]


################################################################################################################################
# Programming Start
################################################################################################################################

# -------------------------------------------------------------------------------------------------------------------------------
# Import Libraries
# -------------------------------------------------------------------------------------------------------------------------------
# Custom Libraries
from Nucleon import Gluonix

# Default Libraries
import os
import time
import atexit
import datetime
import warnings

# -------------------------------------------------------------------------------------------------------------------------------
# Customization
# -------------------------------------------------------------------------------------------------------------------------------
warnings.filterwarnings("ignore", category=DeprecationWarning)

# -------------------------------------------------------------------------------------------------------------------------------
# Global Variables
# -------------------------------------------------------------------------------------------------------------------------------
Error_List = []
Log_Folder = "./Log"
Log_File = "Gluonix_Error.log"
Error_Log = True
Root = False

# -------------------------------------------------------------------------------------------------------------------------------
# Error Log
# -------------------------------------------------------------------------------------------------------------------------------
def Error(E):
    global Error_List, Error_Display
    Error_Time = int(time.time())
    Error_List.append([E, Error_Time])
    if Error_Log:
        if not os.path.exists(Log_Folder):
            os.makedirs(Log_Folder)
        Log_Path = os.path.join(Log_Folder, Log_File)
        Date = datetime.datetime.fromtimestamp(Error_Time)
        Date = Date.strftime("%Y-%m-%d %H:%M:%S")
        Line = f"{Date} - {E}"
        File_Content = []
        if os.path.exists(Log_Path):
            with open(Log_Path, "r") as File:
                File_Content = File.readlines()
            if len(File_Content) >= 10000:
                File_Content.pop()
        with open(Log_Path, "w") as File:
            File.write(Line.rstrip("\r\n") + "\n")
            File.writelines(File_Content)
            
# -------------------------------------------------------------------------------------------------------------------------------
# Relative Path
# -------------------------------------------------------------------------------------------------------------------------------

def File(Name):
    try:
        Base_Dir = os.path.dirname(os.path.abspath(__file__))
        Path = os.path.join(Base_Dir, "Data", "File", f"{Name}")
        if os.path.exists(Path):
            return Path
        else:
            return ""
    except Exception as E:
        Error("Image -> " + str(E))

def Data(Name):
    try:
        Base_Dir = os.path.dirname(os.path.abspath(__file__))
        Path = os.path.join(Base_Dir, "Data", f"{Name}")
        if os.path.exists(Path):
            return Path
        else:
            return ""
    except Exception as E:
        Error("Data -> " + str(E))
        
# -------------------------------------------------------------------------------------------------------------------------------
# Database
# -------------------------------------------------------------------------------------------------------------------------------
Database = Gluonix.SQL(Data('NGD.dll'))

# -------------------------------------------------------------------------------------------------------------------------------
# Database Variables
# -------------------------------------------------------------------------------------------------------------------------------
Variable_Data = {}
Variable_Data_Temp = Database.Get("SELECT * FROM `Variable`", Keys=True)
for Each in Variable_Data_Temp:
        Variable_Data[Each['ID']] = Each['Data']

try:
    Error_Log = bool(int(Variable_Data['Error_Log']))
except:
    Error_Log = True

# -------------------------------------------------------------------------------------------------------------------------------
# Global Functions
# -------------------------------------------------------------------------------------------------------------------------------

def Error_Clear():
    try:
        global Error_List
        Error_List = []
    except Exception as E:
        Error("Error_Clear -> " + str(E))
        
def Load(Global):
    try:
        Display = Database.Get(f"SELECT * FROM `Display` WHERE `ID`='Root'", Keys=True)
        if len(Display)==1:
            Display = Display[0]
            Global[Display['ID']] = Gluonix.GUI()
            Temp_Root = Global[Display['ID']]
            Temp_Root.Config(Error_Log=Error_Log)
            Temp_Root.Config(Width=int(Display['Width']), Height=int(Display['Height']), Left=int(Display['Left']), Top=int(Display['Top']))
            Temp_Root.Config(Title=Display['Title'], Background=Display['Background'], Icon=File(Variable_Data['Icon']), Resizable=bool(int(Display['Resizable'])))
            Temp_Root.Config(Persistent=bool(int(Display['Persistent'])), Full_Screen=bool(int(Display['Full_Screen'])), Toolbar=bool(int(Display['Toolbar'])), Alignment=Display['Alignment'], Menu_Enable=bool(int(Display['Menu'])))
            Temp_Root.Create()
            Temp_Root.Hide()
            Load_Child(Display['ID'], Temp_Root, Global)
    except Exception as E:
        Error("Load -> "+str(E))
        
def Load_Child(Parent, Root, Global):
    try:
        Widgets = Database.Get(f"SELECT * FROM `Widget` WHERE `Root`='{Parent}'", Keys=True)
        for Widget in Widgets:
            if Widget['Type']=='Image':
                if bool(Widget['Interactive']):
                    Temp_Class = getattr(Gluonix, "Image_Zoom")
                else:
                    Temp_Class = getattr(Gluonix, f"{Widget['Type']}")
            else:
                Temp_Class = getattr(Gluonix, f"{Widget['Type']}")
            Temp_Widget = Temp_Class(Root)
            if Widget['Alignment']=='Percentage':
                Fixture = Root.Locate(Widget['Width'], Widget['Height'], Widget['Left'], Widget['Top'])
            else:
                Fixture = [Widget['Width'], Widget['Height'], Widget['Left'], Widget['Top']]
            Temp_Widget.Config(Name=Widget['Name'])
            Temp_Widget.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            if Widget['Background']=='False':
                Temp_Widget.Config(Background=False)
            else:
                Temp_Widget.Config(Background=Widget['Background'])
            Temp_Widget.Config(Foreground=Widget['Foreground'], Border_Size=Widget['Border_Size'], Border_Color=Widget['Border_Color'], Radius=Widget['Radius'], Display=bool(Widget['Display']), Disable=bool(Widget['Disabled']))
            Temp_Widget.Config(Font_Size=Widget['Font_Size'], Font_Weight=Widget['Font_Weight'], Font_Family=Widget['Font_Family'], Align=Widget['Align'], Value=Widget['Value'])
            Temp_Widget.Config(Shadow_Size=Widget['Shadow_Size'], Shadow_Color=Widget['Shadow_Color'], Shadow_Full=Widget['Shadow_Full'])
            Temp_Widget.Config(Resize=True, Move=True)
            Temp_Widget.Config(Resize_Width=bool(Widget['Resize_Width']), Resize_Height=bool(Widget['Resize_Height']))
            Temp_Widget.Config(Move_Left=bool(Widget['Move_Left']), Move_Top=bool(Widget['Move_Top']))
            Temp_Widget.Config(Progress=Widget['Progress'], Zero=Widget['Zero'], Increment=Widget['Increment'], Mimimum=Widget['Minimum'], Maximum=Widget['Maximum'])
            Temp_Widget.Config(Path=File(Widget['ID']))
            Temp_Widget.Config(Url=bool(Widget['Url']), Transparent=bool(Widget['Transparent']), Rotate=Widget['Rotate'], Compound=Widget['Compound'], Aspect_Ratio=bool(Widget['Aspect_Ratio']))
            Temp_Widget.Config(Secure=Widget['Secure'])
            Temp_Widget.Config(Orient=Widget['Orient'], Ridge=Widget['Ridge'], Height_List=Widget['Height_List'])
            Temp_Widget.Create()
        Items = Database.Get(f"SELECT * FROM `Item` WHERE `Root`='{Parent}'", Keys=True)
        for Item in Items:
            Temp_Class = getattr(Gluonix, f"{Item['Type']}")
            Temp_Item = Temp_Class(Root)
            Fixture = [Item['Width'], Item['Height'], Item['Left'], Item['Top']]
            Temp_Item.Config(Name=Item['Name'])
            Temp_Item.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Radius=Item['Radius'])
            Temp_Item.Config(Start=Item['Start'], Extent=Item['Extent'])
            Temp_Item.Config(Size=Item['Size'], Weight=Item['Weight'], Font=Item['Font'], Value=Item['Value'])
            Temp_Item.Config(Thickness=Item['Thickness'], Fill=Item['Fill'], Outline=Item['Outline'])
            Temp_Item.Config(Justify=Item['Justify'], Anchor=Item['Anchor'])
            Temp_Item.Config(Resize=bool(Item['Resize']), Move=bool(Item['Move']))
            Temp_Item.Config(Url=bool(Item['Url']), Array=bool(Item['Array']), Pil=bool(Item['Pil']), Transparent=bool(Item['Transparent']), Rotate=Item['Rotate'], Aspect_Ratio=bool(Item['Aspect_Ratio']))
            Temp_Item.Config(Path=File(Item['ID']))
            Temp_Item.Create()
        Frames = Database.Get(f"SELECT * FROM `Frame` WHERE `Root`='{Parent}'", Keys=True)
        for Frame in Frames:
            Temp_Class = getattr(Gluonix, f"{Frame['Type']}")
            Temp_Frame = Temp_Class(Root)
            if Frame['Alignment']=='Percentage':
                Fixture = Root.Locate(Frame['Width'], Frame['Height'], Frame['Left'], Frame['Top'])
            else:
                Fixture = [Frame['Width'], Frame['Height'], Frame['Left'], Frame['Top']]
            Temp_Frame.Config(Name=Frame['Name'])
            Temp_Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            if Frame['Background']=='False':
                Temp_Frame.Config(Background=False)
            else:
                Temp_Frame.Config(Background=Frame['Background'])
            Temp_Frame.Config(Border_Size=Frame['Border_Size'], Border_Color=Frame['Border_Color'], Radius=Frame['Radius'], Display=bool(Frame['Display']))
            Temp_Frame.Config(Shadow_Size=Frame['Shadow_Size'], Shadow_Color=Frame['Shadow_Color'], Shadow_Full=Frame['Shadow_Full'])
            Temp_Frame.Config(Resize=True, Move=True)
            Temp_Frame.Config(Resize_Width=bool(Frame['Resize_Width']), Resize_Height=bool(Frame['Resize_Height']))
            Temp_Frame.Config(Move_Left=bool(Frame['Move_Left']), Move_Top=bool(Frame['Move_Top']))
            Temp_Frame.Create()
            Load_Child(Frame['ID'], Temp_Frame, Global)
    except Exception as E:
        Error("Load_Child -> "+str(E))
        
def Create_Popup(Display):
    try:
        Global = globals()
        if isinstance(Display, int):
            Display = str(Display)
        if Display.isdigit:
            Display = 'Popup'+Display
        Display = Database.Get(f"SELECT * FROM `Display` WHERE `ID`='{Display}'", Keys=True)
        if len(Display)==1:
            Display = Display[0]
            Temp_Popup = Gluonix.Popup(Global['Root'])
            Temp_Popup.Config(Width=int(Display['Width']), Height=int(Display['Height']), Left=int(Display['Left']), Top=int(Display['Top']))
            Temp_Popup.Config(Title=Display['Title'], Background=Display['Background'], Icon=File(Variable_Data['Icon']), Resizable=bool(int(Display['Resizable'])))
            Temp_Popup.Config(Persistent=bool(int(Display['Persistent'])), Full_Screen=bool(int(Display['Full_Screen'])), Toolbar=bool(int(Display['Toolbar'])), Alignment=Display['Alignment'], Topmost=bool(int(Display['Topmost'])), Menu_Enable=bool(int(Display['Menu'])))
            Temp_Popup.Create()
            Load_Child(Display['ID'], Temp_Popup, Global)
            Temp_Popup.Show()
            return Temp_Popup
    except Exception as E:
        Error("Create_Popup -> "+str(E))
        
def Cleanup():
    try:
        global Database
        Database.Close()
    except Exception as E:
        Error("Close -> "+str(E))
        
atexit.register(Cleanup)

# -------------------------------------------------------------------------------------------------------------------------------
# GUI
# -------------------------------------------------------------------------------------------------------------------------------

Load(globals())
Root.Popup = Create_Popup
Root.Show()


################################################################################################################################
# Programming End
################################################################################################################################


################################################################################################################################
#Main
################################################################################################################################
import inspect

#Program
from .N_Home import Home
from .N_Design import Design

class Main:
    def __init__(self, Global):
        try:
            self.Global = Global
            self.Widget = []
            #Frame
            Fixture = self.Global['GUI'].Locate(100, 100, 0, 0)
            self.Frame = self.Global['Gluonix'].Scroll(self.Global['GUI'])
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Border_Size=0, Scrollbar=20)
            self.Frame.Config(Resize=True, Move=False)
            self.Frame.Create()
            self.Frame.Hide()
            
            #Home Page
            self.Home = Home(self.Global, self)
            
            #Home Page
            self.Design = Design(self.Global, self)
            
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))